from matplotlib import ticker
from isaac_analyzer.logging import getLogger
import matplotlib.pyplot as plt
import numpy as np

from isaac_analyzer.utils import calculate_percentages

logger = getLogger(__name__)


def stacked_bar(
    buckets,
    values_bottom,
    values_upper,
    legend_bottom,
    legend_upper,
    color_bottom,
    color_upper,
    xLabel,
    yLabel,
    title,
    ymajor_ticks=ticker.MultipleLocator(10),
    yminor_ticks=ticker.MultipleLocator(5),
):
    fig, plot = plt.subplots(nrows=1, ncols=1, layout="constrained", figsize=(18, 5))

    percentage_bottom_1, percentage_upper_1 = calculate_percentages(
        values_bottom, values_upper, buckets
    )

    lower_bars_1 = plot.bar(
        buckets, values_bottom, label=legend_bottom, color=color_bottom
    )
    upper_bars_1 = plot.bar(
        buckets,
        values_upper,
        bottom=values_bottom,
        label=legend_upper,
        color=color_upper,
    )

    plot.set_xlabel(xLabel)
    plot.set_ylabel(yLabel)
    plot.legend()
    plot.yaxis.set_minor_locator(yminor_ticks)
    plot.yaxis.set_major_locator(ymajor_ticks)
    plot.tick_params(axis="x", labelrotation=90)
    # for label in plot.get_xticklabels():
    #     label.set(rotation=45)
    logger.debug("Plot1 created")

    for i in range(len(buckets)):
        if values_bottom[i] > 0:
            plot.annotate(
                f"{percentage_bottom_1[i]:.1f}%\n({values_bottom[i]})",
                xy=(
                    lower_bars_1[i].get_x() + lower_bars_1[i].get_width() / 2,
                    values_bottom[i] / 2,
                ),
                xytext=(0, 0),
                textcoords="offset points",
                ha="center",
                va="center",
                color="white",
                fontsize=8,
            )
        if values_upper[i] > 0:
            plot.annotate(
                f"{percentage_upper_1[i]:.1f}%\n({values_upper[i]})",
                xy=(
                    upper_bars_1[i].get_x() + upper_bars_1[i].get_width() / 2,
                    values_bottom[i] + values_upper[i] / 2,
                ),
                xytext=(0, 0),
                textcoords="offset points",
                ha="center",
                va="center",
                color="white",
                fontsize=8,
            )
    logger.debug("Annotations added to plot1")

    fig.suptitle(title, size="x-large", weight="bold")
    logger.debug("Figure created successfully")

    return fig


def hbars(values, labels, x_label, title):
    logger.info("Generating hbar chart")
    fig, plot = plt.subplots(nrows=1, ncols=1, layout="constrained", figsize=(9, 5))

    y_pos = np.arange(len(labels))

    hbars = plot.barh(y_pos, values, align="center")
    plot.bar_label(hbars, label_type="center", fmt="%1.2f", color="white")
    plot.set_yticks(y_pos, labels=labels)
    plot.set_xlabel(x_label)
    plot.xaxis.set_minor_locator(ticker.MultipleLocator(5))
    plot.xaxis.set_major_locator(ticker.MultipleLocator(10))
    fig.suptitle(title, size="x-large", weight="bold")
    return fig


def pie_chart(values, labels, colors, title):
    logger.info("Generating pie chart")
    fig, plot = plt.subplots(nrows=1, ncols=1, layout="constrained", figsize=(9, 5))

    def func(pct, allvals):
        absolute = int(np.round(pct / 100.0 * np.sum(allvals)))
        return f"{pct:.1f}%\n({absolute:d})"

    plot.pie(
        values,
        labels=labels,
        colors=colors,
        autopct=lambda pct: func(pct, values),
        labeldistance=None,
        textprops={"color": "white"},
    )

    plot.legend(loc="upper right", bbox_to_anchor=(1.25, 1))
    fig.suptitle(title, size="x-large", weight="bold")

    logger.debug("Pie chart figure created successfully")
    return fig



from isaac_analyzer.logging import getLogger
from isaac_analyzer.plotter import plot_curses
from isaac_analyzer.utils import add_dicts, get_empty_curses_dict, get_month_header
from os.path import join

logger = getLogger(__name__)


def get_curse_distribution(run):
    logger.debug("Calculating curse distribution for run.")
    curses = get_empty_curses_dict()

    for floor in run["floors"]:
        curse_type = floor.get("curseType", "No Curse")
        curses[curse_type] += 1
        if curse_type != "No Curse":
            curses["Total curses"] += 1

    logger.debug(f"Curse distribution: {curses}")
    return curses


def generate_curses_plot(analyzed_runs, output_path):
    logger.info("Generating curses plot.")
    curses = get_empty_curses_dict()

    for analyzed_run in analyzed_runs:
        for run in analyzed_run["runs"]:
            curses = add_dicts(curses, run["analytics"]["curses"])

    curses_figure = plot_curses(
        curses, title=f"Curses ({get_month_header(analyzed_runs[0]["date"])})"
    )
    output_file = join(output_path, "curses.png")
    curses_figure.savefig(output_file, transparent=True)
    logger.info(f"Curses plot saved to {output_file}")



from isaac_analyzer.analysis.boss import generate_boss_plot, get_boss_type
from isaac_analyzer.analysis.curses import generate_curses_plot, get_curse_distribution
from isaac_analyzer.analysis.endings import generate_picture_type_plot, get_picture_type
from isaac_analyzer.analysis.items import (
    generate_all_items_per_run_taken_plot,
    generate_all_items_per_run_type_plot,
    generate_all_items_plot,
    generate_itemRoom_item_plot,
    get_all_items_statistic,
    get_itemRoom_items_statistics,
)
from isaac_analyzer.analysis.shop import (
    generate_shop_plot,
    get_shop_details,
    get_shop_items,
    generate_shop_item_plot,
)
from isaac_analyzer.logging import getLogger
from isaac_analyzer.run_loader import load_run_file
from isaac_analyzer.analysis.deals import (
    generate_deal_per_run_plot,
    get_deal_type,
    get_deal_chances,
    generate_deal_plot,
)
from os.path import join
from glob import glob
from PIL import Image
import math

logger = getLogger(__name__)


def analyze_run_file(run_file):
    logger.info(f"Analyzing run file with run number {run_file['run_number']}.")
    for run in run_file["runs"]:
        analytics = {
            "deals": get_deal_type(run),
            "dealChance": get_deal_chances(run),
            "curses": get_curse_distribution(run),
            "shops": get_shop_details(run),
            "shop_items": get_shop_items(run),
            "itemRoom_items": get_itemRoom_items_statistics(run),
            "all_items": get_all_items_statistic(run),
            "picture_type": get_picture_type(run),
            "bosses": get_boss_type(run),
        }
        run["analytics"] = analytics
    logger.debug(f"Run file analysis complete: {run_file}")
    return run_file


def analyze_single_run(file_path, output_path):
    logger.info(f"Analyzing single run from file: {file_path}")
    run_file = load_run_file(file_path)
    analyzed_run_file = analyze_run_file(run_file)
    logger.info(f"Single run analysis complete. Results: {analyzed_run_file}")
    logger.warn(analyze_run_file["analytics"])


def analyze_runs(directory_path, output_path):
    logger.info(f"Analyzing all runs in directory: {directory_path}")
    yaml_files = glob(join(directory_path, "*.y*ml"))
    analyzed_runs = []

    for yaml_file in yaml_files:
        logger.info(f"Loading run file: {yaml_file}")
        run_file = load_run_file(yaml_file)
        analyzed_run = analyze_run_file(run_file)
        analyzed_runs.append(analyzed_run)
        logger.info(f"Run file {yaml_file} analyzed.")

    logger.info("All run files analyzed. Generating plots.")
    generate_plots(analyzed_runs, output_path)
    logger.info("Plot generation complete.")


def generate_plots(analyzed_runs, output_path):
    generate_deal_plot(analyzed_runs, output_path)
    generate_curses_plot(analyzed_runs, output_path)
    generate_shop_plot(analyzed_runs, output_path)
    generate_all_items_plot(analyzed_runs, output_path)
    generate_shop_item_plot(analyzed_runs, output_path)
    generate_itemRoom_item_plot(analyzed_runs, output_path)
    generate_picture_type_plot(analyzed_runs, output_path)
    generate_boss_plot(analyzed_runs, output_path)
    generate_all_items_per_run_taken_plot(analyzed_runs, output_path)
    generate_all_items_per_run_type_plot(analyzed_runs, output_path)
    generate_deal_per_run_plot(analyzed_runs, output_path)
    # combine_plots(output_path)
    combine_images_2_column_grid(
        [
            "deals.png",
            "curses.png",
            "shops.png",
            "shop_items.png",
            "itemRoom_items.png",
            "all_items.png",
            "picture_type.png",
            "boss.png",
        ],
        output_path,
    )
    combine_images_vertically(
        [
            "grid.png",
            "all_items_per_run_taken.png",
            "all_items_per_run_type.png",
            "deals_per_run.png",
        ],
        output_path,
    )


def combine_images_2_column_grid(image_files, output_path):
    # Open all images and determine the size for the grid
    images = [Image.open(join(output_path, image_file)) for image_file in image_files]
    widths, heights = zip(*(image.size for image in images))

    # Calculate the grid dimensions
    max_width = max(widths)
    max_height = max(heights)
    num_images = len(images)
    num_rows = math.ceil(num_images / 2)

    # Create a new blank image with a white background
    grid_width = 2 * max_width
    grid_height = num_rows * max_height
    grid_image = Image.new("RGBA", (grid_width, grid_height), (255, 0, 0, 0))

    # Paste images into the grid
    for index, image in enumerate(images):
        x_offset = (index % 2) * max_width
        y_offset = (index // 2) * max_height
        grid_image.paste(image, (x_offset, y_offset))

    # Save the combined image
    grid_image.save(join(output_path, "grid.png"))


def combine_images_vertically(image_files, output_path):
    # Open all images and get their sizes
    images = [Image.open(join(output_path, image_file)) for image_file in image_files]
    widths, heights = zip(*(img.size for img in images))

    total_height = sum(heights)
    max_width = max(widths)

    # Create a new image with the total width and max height
    combined_image = Image.new("RGBA", (max_width, total_height), (255, 0, 0, 0))

    # Paste each image into the new combined image
    y_offset = 0
    for img in images:
        combined_image.paste(img, (0, y_offset))
        y_offset += img.height

    combined_image.save(join(output_path, "isaac_statistics_transparent.png"))
    combined_image_bg = Image.new("RGBA", (max_width, total_height), "WHITE")
    combined_image_bg.paste(combined_image, (0, 0), combined_image)
    combined_image_bg.save(join(output_path, "isaac_statistics.png"))



from matplotlib import ticker
import numpy as np
from isaac_analyzer.logging import getLogger
import matplotlib.pyplot as plt

from isaac_analyzer.utils import calculate_percentages

logger = getLogger(__name__)


def pie_hbar_chart(
    values_1,
    labels_1,
    colors_1,
    title_1,
    values_2,
    labels_2,
    x_label_2,
    title_2,
    title,
):
    fig, plots = plt.subplots(nrows=1, ncols=2, layout="constrained", figsize=(9, 5))

    ### Plot1
    plot_1 = plots[0]
    logger.debug("Creating plot1")

    def func(pct, allvals):
        absolute = int(np.round(pct / 100.0 * np.sum(allvals)))
        return f"{pct:.1f}%\n({absolute:d})"

    plot_1.pie(
        values_1,
        labels=labels_1,
        colors=colors_1,
        autopct=lambda pct: func(pct, values_1),
        labeldistance=None,
        textprops={"color": "white"},
    )

    plot_1.set_title(title_1)
    plot_1.legend(loc="upper right")
    logger.debug("plot2 created")

    fig.suptitle(title, size="x-large", weight="bold")
    logger.debug("Figure created successfully")

    plot_2 = plots[1]

    y_pos = np.arange(len(labels_2))

    hbars = plot_2.barh(y_pos, values_2, align="center")
    plot_2.bar_label(hbars, label_type="center", fmt="%1.2f", color="white")
    plot_2.set_yticks(y_pos, labels=labels_2)
    plot_2.set_xlabel(x_label_2)
    plot_2.set_xlim(0, 4)
    plot_2.set_title(title_2)

    return fig


def stacked_bar_pie_chart(
    buckets_1,
    values_bottom_1,
    values_upper_1,
    legend_bottom_1,
    legend_upper_1,
    color_bottom_1,
    color_upper_1,
    xLabel_1,
    yLabel_1,
    title_1,
    values_2,
    labels_2,
    colors_2,
    title_2,
    title,
    ymajor_ticks=ticker.MultipleLocator(10),
    yminor_ticks=ticker.MultipleLocator(5),
):
    fig, plots = plt.subplots(nrows=1, ncols=2, layout="constrained", figsize=(9, 5))

    percentage_bottom_1, percentage_upper_1 = calculate_percentages(
        values_bottom_1, values_upper_1, buckets_1
    )

    # Plot1
    plot_1 = plots[0]
    logger.debug("Creating plot1")

    lower_bars_1 = plot_1.bar(
        buckets_1, values_bottom_1, label=legend_bottom_1, color=color_bottom_1
    )
    upper_bars_1 = plot_1.bar(
        buckets_1,
        values_upper_1,
        bottom=values_bottom_1,
        label=legend_upper_1,
        color=color_upper_1,
    )

    plot_1.set_xlabel(xLabel_1)
    plot_1.set_ylabel(yLabel_1)
    plot_1.set_title(title_1)
    plot_1.yaxis.set_minor_locator(yminor_ticks)
    plot_1.yaxis.set_major_locator(ymajor_ticks)
    plot_1.legend()
    logger.debug("Plot1 created")

    for i in range(len(buckets_1)):
        if values_bottom_1[i] > 0:
            plot_1.annotate(
                f"{percentage_bottom_1[i]:.1f}%",
                xy=(
                    lower_bars_1[i].get_x() + lower_bars_1[i].get_width() / 2,
                    values_bottom_1[i] / 2,
                ),
                xytext=(0, 0),
                textcoords="offset points",
                ha="center",
                va="center",
                color="white",
            )
        if values_upper_1[i] > 0:
            plot_1.annotate(
                f"{percentage_upper_1[i]:.1f}%",
                xy=(
                    upper_bars_1[i].get_x() + upper_bars_1[i].get_width() / 2,
                    values_bottom_1[i] + values_upper_1[i] / 2,
                ),
                xytext=(0, 0),
                textcoords="offset points",
                ha="center",
                va="center",
                color="white",
            )
    logger.debug("Annotations added to plot1")

    ### Plot2
    ### Deals plot
    plot_2 = plots[1]
    logger.debug("Creating plot2")

    def func(pct, allvals):
        absolute = int(np.round(pct / 100.0 * np.sum(allvals)))
        return f"{pct:.1f}%\n({absolute:d})"

    plot_2.pie(
        values_2,
        labels=labels_2,
        colors=colors_2,
        autopct=lambda pct: func(pct, values_2),
        labeldistance=None,
        textprops={"color": "white"},
    )

    plot_2.set_title(title_2)
    plot_2.legend(loc="upper right")
    logger.debug("plot2 created")

    fig.suptitle(title, size="x-large", weight="bold")
    logger.debug("Figure created successfully")

    return fig


def double_stacked_bar_chart(
    buckets_1,
    values_bottom_1,
    values_upper_1,
    legend_bottom_1,
    legend_upper_1,
    color_bottom_1,
    color_upper_1,
    xLabel_1,
    yLabel_1,
    title_1,
    buckets_2,
    values_bottom_2,
    values_upper_2,
    legend_bottom_2,
    legend_upper_2,
    color_bottom_2,
    color_upper_2,
    xLabel_2,
    yLabel_2,
    title_2,
    title,
    ymajor_ticks=ticker.MultipleLocator(10),
    yminor_ticks=ticker.MultipleLocator(5),
    ymajor2_ticks=ticker.MultipleLocator(10),
    yminor2_ticks=ticker.MultipleLocator(5),
):
    logger.info("Generating double stacked bar chart")

    fig, plots = plt.subplots(nrows=1, ncols=2, layout="constrained", figsize=(9, 5))

    percentage_bottom_1, percentage_upper_1 = calculate_percentages(
        values_bottom_1, values_upper_1, buckets_1
    )
    percentage_bottom_2, percentage_upper_2 = calculate_percentages(
        values_bottom_2, values_upper_2, buckets_2
    )

    # Plot1
    plot_1 = plots[0]
    logger.debug("Creating plot1")

    lower_bars_1 = plot_1.bar(
        buckets_1, values_bottom_1, label=legend_bottom_1, color=color_bottom_1
    )
    upper_bars_1 = plot_1.bar(
        buckets_1,
        values_upper_1,
        bottom=values_bottom_1,
        label=legend_upper_1,
        color=color_upper_1,
    )

    plot_1.set_xlabel(xLabel_1)
    plot_1.set_ylabel(yLabel_1)
    plot_1.set_title(title_1)
    plot_1.yaxis.set_minor_locator(yminor_ticks)
    plot_1.yaxis.set_major_locator(ymajor_ticks)
    plot_1.legend()
    logger.debug("Plot1 created")

    for i in range(len(buckets_1)):
        if values_bottom_1[i] > 0:
            plot_1.annotate(
                f"{percentage_bottom_1[i]:.1f}%",
                xy=(
                    lower_bars_1[i].get_x() + lower_bars_1[i].get_width() / 2,
                    values_bottom_1[i] / 2,
                ),
                xytext=(0, 0),
                textcoords="offset points",
                ha="center",
                va="center",
                color="white",
            )
        if values_upper_1[i] > 0:
            plot_1.annotate(
                f"{percentage_upper_1[i]:.1f}%",
                xy=(
                    upper_bars_1[i].get_x() + upper_bars_1[i].get_width() / 2,
                    values_bottom_1[i] + values_upper_1[i] / 2,
                ),
                xytext=(0, 0),
                textcoords="offset points",
                ha="center",
                va="center",
                color="white",
            )
    logger.debug("Annotations added to plot1")

    # Plot2
    plot_2 = plots[1]
    logger.debug("Creating plot2")

    lower_bars_2 = plot_2.bar(
        buckets_2, values_bottom_2, label=legend_bottom_2, color=color_bottom_2
    )
    upper_bars_2 = plot_2.bar(
        buckets_2,
        values_upper_2,
        bottom=values_bottom_2,
        label=legend_upper_2,
        color=color_upper_2,
    )

    plot_2.set_xlabel(xLabel_2)
    plot_2.set_ylabel(yLabel_2)
    plot_2.set_title(title_2)
    plot_2.yaxis.set_minor_locator(yminor2_ticks)
    plot_2.yaxis.set_major_locator(ymajor2_ticks)
    plot_2.legend()
    logger.debug("Plot2 created")

    for i in range(len(buckets_2)):
        if values_bottom_2[i] > 0:
            plot_2.annotate(
                f"{percentage_bottom_2[i]:.1f}%",
                xy=(
                    lower_bars_2[i].get_x() + lower_bars_2[i].get_width() / 2,
                    values_bottom_2[i] / 2,
                ),
                xytext=(0, 0),
                textcoords="offset points",
                ha="center",
                va="center",
                color="white",
            )
        if values_upper_2[i] > 0:
            plot_2.annotate(
                f"{percentage_upper_2[i]:.1f}%",
                xy=(
                    upper_bars_2[i].get_x() + upper_bars_2[i].get_width() / 2,
                    values_bottom_2[i] + values_upper_2[i] / 2,
                ),
                xytext=(0, 0),
                textcoords="offset points",
                ha="center",
                va="center",
                color="white",
            )
    logger.debug("Annotations added to plot2")

    fig.suptitle(title, size="x-large", weight="bold")
    logger.debug("Figure created successfully")

    return fig



import yaml
from importlib.resources import files
from isaac_analyzer.logging import getLogger

logger = getLogger(__name__)


def load_yaml(file_path):
    """Load YAML file into a dictionary from a given path."""
    logger.debug(f"Loading YAML file from path: {file_path}")
    try:
        with open(file_path, "r") as file:
            data = yaml.safe_load(file)
        logger.debug(f"Successfully loaded YAML file: {file_path}")
    except Exception as e:
        logger.exception(f"Failed to load YAML file from path: {file_path}, Error: {e}")
    return data


def load_yaml_resource(name):
    """Load YAML file into a dictionary from a resource name."""
    logger.debug(f"Loading YAML resource: {name}")
    try:
        with files("isaac_analyzer").joinpath(f"resources/{name}").open() as file:
            data = yaml.safe_load(file)
        logger.debug(f"Successfully loaded YAML resource: {name}")
    except Exception as e:
        logger.exception(f"Failed to load YAML resource: {name}, Error: {e}")
    return data



from tabulate import tabulate
from isaac_analyzer.logging import getLogger
from isaac_analyzer.yaml_loader import load_yaml_resource

logger = getLogger(__name__)

item_list = []


def load_items():
    global item_list
    if len(item_list) == 0:
        logger.debug("Loading items from items.yaml")
        data = load_yaml_resource("items.yaml")
        item_list = data.get("items", [])
        logger.debug(f"{len(item_list)} items loaded from items.yaml")
    else:
        logger.debug("Items already loaded, using cached list")
    return item_list


def print_items():
    """Print items from items.yaml"""
    logger.debug("Printing all items from items.yaml")
    items = load_items()
    if isinstance(items, list) and all(isinstance(item, dict) for item in items):
        table = tabulate(items, headers="keys", tablefmt="github")
        logger.debug("Items formatted into table")
        print(table)
    else:
        logger.warning("Items list is not in the expected format")


def get_by_id(id):
    logger.debug(f"Getting item with id: {id}")
    items = load_items()
    item = next((item.copy() for item in items if item["id"] == str(id)), None)
    if item:
        logger.debug(f"Item found: {item}")
    else:
        logger.warning(f"No item found with id: {id}")
    return item



from isaac_analyzer.logging import getLogger
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.patches import ConnectionPatch

logger = getLogger(__name__)


def plot_curses(curses: dict, title):
    fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(9, 5))
    fig.subplots_adjust(wspace=0)

    total_curses = curses["Total curses"]
    overall = [curses["Total curses"], curses["No Curse"]]
    labels = ["Curse", "No Curse"]
    explode = [0.1, 0]
    angle = -45

    def func(pct, allvals):
        absolute = int(np.round(pct / 100.0 * np.sum(allvals)))
        return f"{pct:.1f}%\n({absolute:d})"

    wedges, *_ = ax1.pie(
        overall,
        autopct=lambda pct: func(pct, overall),
        startangle=angle,
        labels=labels,
        explode=explode,
        colors=["darkred", "darkgreen"],
        textprops={"color": "white"},
    )

    ax1.legend(loc="upper left")

    del curses["No Curse"]
    del curses["Total curses"]

    detail_values = [curses[key] / total_curses for key in curses.keys()]
    detail_labels = curses.keys()
    bottom = 1
    width = 0.2

    # Adding from the top matches the legend.
    for j, (height, label) in enumerate(reversed([*zip(detail_values, detail_labels)])):
        if height > 0:
            bottom -= height
            bc = ax2.bar(
                0,
                height,
                width,
                bottom=bottom,
                color="darkred",
                label=label,
                alpha=0.125 * j,
            )
            ax2.bar_label(
                bc, labels=[f"{height:.0%}"], label_type="center", color="white"
            )

    ax2.set_title("Curse Type")
    ax2.legend(loc="upper right", bbox_to_anchor=(1.25, 1))
    ax2.axis("off")
    ax2.set_xlim(-2.5 * width, 2.5 * width)

    # use ConnectionPatch to draw lines between the two plots
    theta1, theta2 = wedges[0].theta1, wedges[0].theta2
    center, r = wedges[0].center, wedges[0].r
    bar_height = sum(detail_values)

    # draw top connecting line
    x = r * np.cos(np.pi / 180 * theta2) + center[0]
    y = r * np.sin(np.pi / 180 * theta2) + center[1]
    con = ConnectionPatch(
        xyA=(-width / 2, bar_height),
        coordsA=ax2.transData,
        xyB=(x, y),
        coordsB=ax1.transData,
    )
    con.set_color([0, 0, 0])
    con.set_linewidth(4)
    ax2.add_artist(con)

    # draw bottom connecting line
    x = r * np.cos(np.pi / 180 * theta1) + center[0]
    y = r * np.sin(np.pi / 180 * theta1) + center[1]
    con = ConnectionPatch(
        xyA=(-width / 2, 0), coordsA=ax2.transData, xyB=(x, y), coordsB=ax1.transData
    )
    con.set_color([0, 0, 0])
    ax2.add_artist(con)
    con.set_linewidth(4)

    fig.suptitle(title, size="x-large", weight="bold")
    return fig



from matplotlib import ticker
from isaac_analyzer.logging import getLogger
from isaac_analyzer.utils import (
    add_dicts,
    add_item_analysis_dicts,
    get_empty_item_analysis_dict,
    get_empty_itemRoom_items_dict,
    get_month_header,
    sort_keys_per_day_dict,
)
from isaac_analyzer.plotting.double_chart import (
    double_stacked_bar_chart,
    pie_hbar_chart,
)
from isaac_analyzer.plotting.single_chart import stacked_bar
from os.path import join

logger = getLogger(__name__)


def analyze_items_list(items):
    logger.debug("Analyzing items list")
    item_stats = get_empty_item_analysis_dict()

    for item in items:
        status = "taken" if item["taken"] else "ignored"
        item_quality = str(item["quality"])
        item_type = item["type"]

        item_stats["quality"][item_quality][status] += 1
        item_stats["type"][item_type][status] += 1

    logger.debug("Items list analysis complete")
    return item_stats


def get_itemRoom_items_statistics(run):
    logger.info("Generating itemRoom item statistics for a run")
    itemRoom_items = get_empty_itemRoom_items_dict()

    for floor in run["floors"]:
        itemRoom = floor.get("itemRoom")
        if itemRoom and itemRoom.get("visited"):
            items = itemRoom.get("items", [])
            if items:
                all_quality = sum(item["quality"] for item in items)
                all_count = len(items)
                taken_items = [item for item in items if item["taken"]]
                ignored_items = [item for item in items if not item["taken"]]

                taken_count = len(taken_items)
                ignored_count = all_count - taken_count
                taken_quality = sum(item["quality"] for item in taken_items)
                ignored_quality = sum(item["quality"] for item in ignored_items)

                for item in items:
                    itemRoom_items["type"][item["type"]] += 1

                if all_count > 0:
                    itemRoom_items["quality"]["all"] = (
                        (itemRoom_items["quality"]["all"] + (all_quality / all_count))
                        / 2
                        if itemRoom_items["quality"]["all"] > 0
                        else (all_quality / all_count)
                    )
                if taken_count > 0:
                    itemRoom_items["quality"]["taken"] = (
                        (
                            itemRoom_items["quality"]["taken"]
                            + (taken_quality / taken_count)
                        )
                        / 2
                        if itemRoom_items["quality"]["taken"] > 0
                        else (taken_quality / taken_count)
                    )
                if ignored_count > 0:
                    itemRoom_items["quality"]["ignored"] = (
                        (
                            itemRoom_items["quality"]["ignored"]
                            + (ignored_quality / ignored_count)
                        )
                        / 2
                        if itemRoom_items["quality"]["ignored"] > 0
                        else (ignored_quality / ignored_count)
                    )

    logger.debug("ItemRoom items statistics generation complete")
    return itemRoom_items


def get_all_items_statistic(run):
    logger.info("Generating item statistics for a run")
    item_stats = get_empty_item_analysis_dict()

    for floor in run["floors"]:
        logger.debug(f"Processing floor: {floor}")
        if "itemRoom" in floor and floor["itemRoom"]["visited"]:
            item_stats = add_item_analysis_dicts(
                item_stats, analyze_items_list(floor["itemRoom"]["items"])
            )

        if "shop" in floor and floor["shop"]["visited"] and "items" in floor["shop"]:
            item_stats = add_item_analysis_dicts(
                item_stats, analyze_items_list(floor["shop"]["items"])
            )

        if "items" in floor:
            item_stats = add_item_analysis_dicts(
                item_stats, analyze_items_list(floor["items"])
            )

        if (
            "bossRoom" in floor
            and floor["bossRoom"]["visited"]
            and "items" in floor["bossRoom"]
        ):
            item_stats = add_item_analysis_dicts(
                item_stats, analyze_items_list(floor["bossRoom"]["items"])
            )

        if "specialRooms" in floor:
            for special_room in floor["specialRooms"]:
                if special_room["visited"] and "items" in special_room:
                    item_stats = add_item_analysis_dicts(
                        item_stats, analyze_items_list(special_room["items"])
                    )

    logger.info("Item statistics generation complete")
    return item_stats


def generate_all_items_per_run_taken_plot(analyzed_runs, output_path):
    stats = {}

    for analyzed_run in analyzed_runs:
        logger.debug("Processing analyzed run")
        for id, run in enumerate(analyzed_run["runs"]):
            stats[f"{analyzed_run["date"]}-{id}"] = [
                run["analytics"]["all_items"]["type"]["active"]["taken"]
                + run["analytics"]["all_items"]["type"]["passive"]["taken"],
                run["analytics"]["all_items"]["type"]["active"]["ignored"]
                + run["analytics"]["all_items"]["type"]["passive"]["ignored"],
            ]

    stats = sort_keys_per_day_dict(stats)

    all_items_fig = stacked_bar(
        buckets=stats.keys(),
        values_bottom=[stats[x][0] for x in stats.keys()],
        values_upper=[stats[x][1] for x in stats.keys()],
        legend_bottom="Taken",
        legend_upper="Ignored",
        yLabel="Count",
        xLabel="Run",
        color_bottom="green",
        color_upper="red",
        title=f"All Items per Run ({get_month_header(analyzed_runs[0]["date"])})",
    )
    output_file = join(output_path, "all_items_per_run_taken.png")
    all_items_fig.savefig(output_file, transparent=True)
    logger.info(f"All items plot saved to {output_file}")


def generate_all_items_per_run_type_plot(analyzed_runs, output_path):
    stats = {}

    for analyzed_run in analyzed_runs:
        logger.debug("Processing analyzed run")
        for id, run in enumerate(analyzed_run["runs"]):
            stats[f"{analyzed_run["date"]}-{id}"] = [
                run["analytics"]["all_items"]["type"]["passive"]["taken"]
                + run["analytics"]["all_items"]["type"]["passive"]["ignored"],
                run["analytics"]["all_items"]["type"]["active"]["ignored"]
                + run["analytics"]["all_items"]["type"]["active"]["taken"],
            ]

    stats = sort_keys_per_day_dict(stats)

    all_items_fig = stacked_bar(
        buckets=stats.keys(),
        values_bottom=[stats[x][0] for x in stats.keys()],
        values_upper=[stats[x][1] for x in stats.keys()],
        legend_bottom="Passive",
        legend_upper="Active",
        yLabel="Count",
        xLabel="Run",
        color_bottom="chocolate",
        color_upper="cadetblue",
        title=f"Item Types per Run ({get_month_header(analyzed_runs[0]["date"])})",
    )
    output_file = join(output_path, "all_items_per_run_type.png")
    all_items_fig.savefig(output_file, transparent=True)
    logger.info(f"All items plot saved to {output_file}")


def generate_all_items_plot(analyzed_runs, output_path):
    logger.info("Generating all items plot")
    item_stats = get_empty_item_analysis_dict()

    for analyzed_run in analyzed_runs:
        logger.debug("Processing analyzed run")
        for run in analyzed_run["runs"]:
            item_stats = add_item_analysis_dicts(
                item_stats, run["analytics"]["all_items"]
            )

    all_items_fig = double_stacked_bar_chart(
        buckets_1=list(item_stats["quality"].keys()),
        xLabel_1="Item Quality",
        yLabel_1="Count",
        values_bottom_1=[
            item_stats["quality"][bucket]["taken"] for bucket in item_stats["quality"]
        ],
        values_upper_1=[
            item_stats["quality"][bucket]["ignored"] for bucket in item_stats["quality"]
        ],
        color_bottom_1="green",
        color_upper_1="red",
        title_1="Items taken and ignored by quality",
        legend_bottom_1="Taken",
        legend_upper_1="Ignored",
        buckets_2=[x.capitalize() for x in list(item_stats["type"].keys())],
        xLabel_2="Item Type",
        yLabel_2="Count",
        values_bottom_2=[
            item_stats["type"][bucket]["taken"] for bucket in item_stats["type"]
        ],
        values_upper_2=[
            item_stats["type"][bucket]["ignored"] for bucket in item_stats["type"]
        ],
        color_bottom_2="green",
        color_upper_2="red",
        title_2="Items taken and ignored by type",
        legend_bottom_2="Taken",
        legend_upper_2="Ignored",
        ymajor_ticks=ticker.MultipleLocator(50),
        yminor_ticks=ticker.MultipleLocator(25),
        ymajor2_ticks=ticker.MultipleLocator(100),
        yminor2_ticks=ticker.MultipleLocator(25),
        title=f"All Items ({get_month_header(analyzed_runs[0]["date"])})",
    )

    output_file = join(output_path, "all_items.png")
    all_items_fig.savefig(output_file, transparent=True)
    logger.info(f"All items plot saved to {output_file}")


def generate_itemRoom_item_plot(analyzed_runs, output_path):
    itemRoom_items = get_empty_itemRoom_items_dict()

    for analyzed_run in analyzed_runs:
        logger.debug(f"Processing analyzed run: {analyzed_run}")
        for run in analyzed_run["runs"]:
            logger.debug(f"Processing run: {run}")
            itemRoom_items = {
                "type": add_dicts(
                    itemRoom_items["type"], run["analytics"]["itemRoom_items"]["type"]
                ),
                "quality": add_dicts(
                    itemRoom_items["quality"],
                    run["analytics"]["itemRoom_items"]["quality"],
                ),
            }

    itemRoom_item_figure = pie_hbar_chart(
        values_1=[itemRoom_items["type"][x] for x in itemRoom_items["type"]],
        labels_1=[x.capitalize() for x in itemRoom_items["type"].keys()],
        colors_1=["red", "green"],
        title_1="Item Type",
        values_2=list(
            reversed(
                [
                    itemRoom_items["quality"][x] / len(analyzed_runs)
                    for x in itemRoom_items["quality"]
                ]
            )
        ),
        labels_2=[
            x.capitalize() for x in list(reversed(itemRoom_items["quality"].keys()))
        ],
        x_label_2="Quality",
        title_2="Average Quality of Item Groups",
        title=f"Treasure Room Items ({get_month_header(analyzed_runs[0]["date"])})",
    )

    output_file = join(output_path, "itemRoom_items.png")
    itemRoom_item_figure.savefig(output_file, transparent=True)
    logger.info(f"Shop plot saved to {output_file}")



from argparse import ArgumentParser
from os.path import join
from glob import glob
from logging import DEBUG, INFO
from typing import Sequence

from isaac_analyzer import __version__
from isaac_analyzer.items import print_items, get_by_id
from isaac_analyzer.run_loader import load_run_file
from isaac_analyzer.validator import validate_yaml
from isaac_analyzer.logging import init, getLogger
from isaac_analyzer.analysis.main import analyze_single_run, analyze_runs

logger = getLogger(__name__)


def parse_arguments():
    parser = ArgumentParser(
        description="isaac-analyzer: A CLI application to analyze isaac runs"
    )
    parser.add_argument(
        "--version", action="version", version=f"isaac-analyzer {__version__}"
    )
    parser.add_argument(
        "-v", "--verbose", action="store_true", help="Enable verbose logging"
    )

    subparsers = parser.add_subparsers(dest="command", help="Sub-command help")

    add_items_subparser(subparsers)
    add_validate_subparser(subparsers)
    add_analyze_subparser(subparsers)
    add_dry_run_subparser(subparsers)

    return parser.parse_args()


def add_items_subparser(subparsers: ArgumentParser):
    parser_items = subparsers.add_parser(
        "items", help="Operations related to items.yaml"
    )
    items_subparsers = parser_items.add_subparsers(
        dest="items_command", help="Items command help"
    )

    items_subparsers.add_parser("list", help="Print items from items.yaml")

    parser_get = items_subparsers.add_parser("get", help="Get an item by ID")
    parser_get.add_argument("id", help="The ID of the item to fetch")


def add_validate_subparser(subparsers: ArgumentParser):
    parser_validate = subparsers.add_parser(
        "validate", help="Validate a YAML file against the schema"
    )
    parser_validate.add_argument(
        "-f", "--file", type=str, help="Path to a single YAML file to validate"
    )
    parser_validate.add_argument(
        "-d",
        "--directory",
        type=str,
        help="Path to a directory containing YAML files to validate",
    )


def add_dry_run_subparser(subparsers: ArgumentParser):
    parser_dry_run = subparsers.add_parser(
        "load",
        help="Load a directory of run files to see if the run loader throws any issues",
    )
    parser_dry_run.add_argument(
        "-d",
        "--directory",
        type=str,
        help="Path to a directory containing YAML files to dry load",
    )


def add_analyze_subparser(subparsers: ArgumentParser):
    parser_analyze = subparsers.add_parser(
        "analyze", help="Analyze a given file or directory of runs"
    )
    parser_analyze.add_argument(
        "-f", "--file", type=str, help="Path to a single YAML file to analyze"
    )
    parser_analyze.add_argument(
        "-d",
        "--directory",
        type=str,
        help="Path to a directory containing YAML files to analyze",
    )
    parser_analyze.add_argument(
        "-o",
        "--output",
        type=str,
        help="Path to a directory where images shall be saved",
    )


def setup_logging(verbose: bool):
    log_level = DEBUG if verbose else INFO
    init(log_level)
    if verbose:
        logger.debug("Verbose logging enabled.")


def handle_items_command(args: Sequence[str]):
    if args.items_command == "list":
        print_items()
    elif args.items_command == "get":
        item = get_by_id(args.id)
        if item:
            logger.info(item)
        else:
            logger.error(f"Item with ID {args.id} not found.")


def handle_validate_command(args: Sequence[str]):
    if args.file:
        validate_single_file(args.file)
    elif args.directory:
        validate_directory(args.directory)


def handle_analyze_command(args: Sequence[str]):
    if args.file:
        analyze_single_file(args.file, args.output)
    elif args.directory:
        analyze_directory(args.directory, args.output)


def handle_load_command(args: Sequence[str]):
    logger.info(
        f"Loading runs inside ({args.directory}) with the run_loader to see potential issues."
    )
    yaml_files = glob(join(args.directory, "*.y*ml"))
    all_good = True

    for file in yaml_files:
        try:
            load_run_file(file)
            logger.debug(f"Successfully loaded file: {file}")
        except Exception as e:
            all_good = False
            logger.error(f"Failed loading file: {file} with error: {e}")

    if all_good:
        logger.info(f"All files inside directory ({args.directory}) loadable.")
        exit(0)
    else:
        exit(1)


def validate_single_file(file_path):
    schema_path = join("resources", "run_file_schema.json")
    try:
        validate_yaml(file_path, schema_path)
        logger.info(f"Validation successful for file: {file_path}")
    except Exception as e:
        logger.error(f"Validation failed for file: {file_path} with error: {e}")
        exit(1)


def validate_directory(directory_path: str):
    schema_path = join("resources", "run_file_schema.json")
    yaml_files = glob(join(directory_path, "*.y*ml"))
    all_valid = True

    for yaml_file in yaml_files:
        try:
            validate_yaml(yaml_file, schema_path)
            logger.info(f"Validation successful for file: {yaml_file}")
        except Exception as e:
            all_valid = False
            logger.error(f"Validation failed for file: {yaml_file} with error: {e}")

    if all_valid:
        logger.info("All YAML files are valid.")
    else:
        logger.error("Some YAML files failed validation. See error messages above.")
        exit(1)


def analyze_single_file(file_path, output_path):
    logger.info(f"Analyzing run file: {file_path}")
    analyze_single_run(file_path, output_path)


def analyze_directory(directory_path, output_path):
    logger.info(f"Analyzing all runs in directory: {directory_path}")
    analyze_runs(directory_path, output_path)


def main():
    args = parse_arguments()
    setup_logging(args.verbose)

    if args.command == "items":
        handle_items_command(args)
    elif args.command == "validate":
        handle_validate_command(args)
    elif args.command == "analyze":
        handle_analyze_command(args)
    elif args.command == "load":
        handle_load_command(args)
    else:
        logger.error("Invalid command. Use --help for more information.")


if __name__ == "__main__":
    main()



from isaac_analyzer.logging import getLogger
from isaac_analyzer.plotting.single_chart import pie_chart
from isaac_analyzer.plotting.double_chart import pie_hbar_chart
from isaac_analyzer.utils import (
    add_dicts,
    get_empty_shop_details_dict,
    get_empty_shop_items_dict,
    get_month_header,
)
from os.path import join

logger = getLogger(__name__)


def get_shop_items(run):
    shop_items = get_empty_shop_items_dict()

    for floor in run["floors"]:
        shop = floor.get("shop")
        if shop and shop.get("visited") and "boss" not in shop:
            items = shop.get("items", [])
            if items:
                all_quality = sum(item["quality"] for item in items)
                all_count = len(items)
                taken_items = [item for item in items if item["taken"]]
                ignored_items = [item for item in items if not item["taken"]]

                taken_count = len(taken_items)
                ignored_count = all_count - taken_count
                taken_quality = sum(item["quality"] for item in taken_items)
                ignored_quality = sum(item["quality"] for item in ignored_items)

                if taken_count > 0:
                    shop_items["shop_usage"]["used"] += 1
                else:
                    shop_items["shop_usage"]["ignored"] += 1

                if all_count > 0:
                    shop_items["quality"]["all"] = (
                        (shop_items["quality"]["all"] + (all_quality / all_count)) / 2
                        if shop_items["quality"]["all"] > 0
                        else (all_quality / all_count)
                    )
                if taken_count > 0:
                    shop_items["quality"]["taken"] = (
                        (shop_items["quality"]["taken"] + (taken_quality / taken_count))
                        / 2
                        if shop_items["quality"]["taken"] > 0
                        else (taken_quality / taken_count)
                    )
                if ignored_count > 0:
                    shop_items["quality"]["ignored"] = (
                        (
                            shop_items["quality"]["ignored"]
                            + (ignored_quality / ignored_count)
                        )
                        / 2
                        if shop_items["quality"]["ignored"] > 0
                        else (ignored_quality / ignored_count)
                    )

    return shop_items


def get_shop_details(run):
    logger.debug("Calculating shop greed and visiting rate.")
    shops = get_empty_shop_details_dict()

    for floor in run["floors"]:
        if "shop" in floor:
            if floor["shop"]["visited"]:
                if "boss" in floor["shop"]:
                    shops["visited_boss"] += 1
                else:
                    shops["visited"] += 1
            else:
                shops["skipped"] += 1

    logger.debug(f"Shop details: {shops}")
    return shops


def generate_shop_plot(analyzed_runs, output_path):
    logger.info("Generating shop plot.")
    shops = get_empty_shop_details_dict()

    for analyzed_run in analyzed_runs:
        for run in analyzed_run["runs"]:
            shops = add_dicts(shops, run["analytics"]["shops"])

    values = [shops[key] for key in shops.keys()]
    labels = ["Greed", "Normal", "Skipped"]
    colors = ["steelblue", "goldenrod", "red"]
    shop_figure = pie_chart(
        values, labels, colors, f"Shops ({get_month_header(analyzed_runs[0]["date"])})"
    )

    output_file = join(output_path, "shops.png")
    shop_figure.savefig(output_file, transparent=True)
    logger.info(f"Shop plot saved to {output_file}")


def generate_shop_item_plot(analyzed_runs, output_path):
    shop_items = get_empty_shop_items_dict()

    for analyzed_run in analyzed_runs:
        for run in analyzed_run["runs"]:
            shop_items = {
                "shop_usage": add_dicts(
                    shop_items["shop_usage"],
                    run["analytics"]["shop_items"]["shop_usage"],
                ),
                "quality": add_dicts(
                    shop_items["quality"], run["analytics"]["shop_items"]["quality"]
                ),
            }

    shop_item_figure = pie_hbar_chart(
        values_1=[shop_items["shop_usage"][x] for x in shop_items["shop_usage"]],
        labels_1=["Taken", "Ignored"],
        colors_1=["green", "red"],
        title_1="Shop Usage",
        values_2=list(
            reversed(
                [
                    shop_items["quality"][x] / len(analyzed_runs)
                    for x in shop_items["quality"]
                ]
            )
        ),
        labels_2=[x.capitalize() for x in list(reversed(shop_items["quality"].keys()))],
        x_label_2="Quality",
        title_2="Average Quality of Item Groups",
        title=f"Shop Items ({get_month_header(analyzed_runs[0]["date"])})",
    )

    output_file = join(output_path, "shop_items.png")
    shop_item_figure.savefig(output_file, transparent=True)
    logger.info(f"Shop plot saved to {output_file}")



from matplotlib import ticker
from isaac_analyzer.logging import getLogger
from isaac_analyzer.plotting.single_chart import stacked_bar
from isaac_analyzer.utils import (
    get_empty_deal_chance_dict,
    get_month_header,
    sort_keys_per_day_dict,
)
from isaac_analyzer.plotting.double_chart import stacked_bar_pie_chart
from os.path import join

logger = getLogger(__name__)


def get_deal_chances(run):
    logger.debug("Calculating deal chances for run.")
    deal_buckets = get_empty_deal_chance_dict()

    for floor in run["floors"]:
        if "deal" in floor:
            chance = floor["deal"]["chance"]
            deal_type_present = bool(floor["deal"]["type"])
            bucket = (
                "<0.25"
                if chance <= 0.25
                else "<0.50"
                if chance <= 0.50
                else "<0.75"
                if chance <= 0.75
                else ">0.75"
            )
            deal_buckets[bucket]["hit" if deal_type_present else "miss"] += 1

    logger.debug(f"Deal chances calculated: {deal_buckets}")
    return deal_buckets


def get_deal_type(run):
    logger.debug("Calculating deal types for run.")
    deals = {"total": 0, "angel": 0, "devil": 0}

    for floor in run["floors"]:
        if "deal" in floor and floor["deal"]["type"]:
            deal_type = floor["deal"]["type"]
            deals[deal_type] += 1
            deals["total"] += 1

    logger.debug(f"Deal types calculated: {deals}")
    return deals


def generate_deal_per_run_plot(analyzed_runs, output_path):
    stats = {}

    for analyzed_run in analyzed_runs:
        logger.debug("Processing analyzed run")
        for id, run in enumerate(analyzed_run["runs"]):
            stats[f"{analyzed_run["date"]}-{id}"] = [
                run["analytics"]["deals"]["devil"],
                run["analytics"]["deals"]["angel"],
            ]

    stats = sort_keys_per_day_dict(stats)

    all_items_fig = stacked_bar(
        buckets=stats.keys(),
        values_bottom=[stats[x][0] for x in stats.keys()],
        values_upper=[stats[x][1] for x in stats.keys()],
        legend_bottom="Devil",
        legend_upper="Angel",
        yLabel="Count",
        xLabel="Run",
        color_bottom="dimgray",
        color_upper="gold",
        ymajor_ticks=ticker.MultipleLocator(5),
        yminor_ticks=ticker.MultipleLocator(1),
        title=f"Devil and Angel Deals ({get_month_header(analyzed_runs[0]["date"])})",
    )
    output_file = join(output_path, "deals_per_run.png")
    all_items_fig.savefig(output_file, transparent=True)
    logger.info(f"All items plot saved to {output_file}")


def generate_deal_plot(analyzed_runs, output_path):
    logger.info("Generating deal plot.")
    deal_count = [0, 0]
    deal_chances = get_empty_deal_chance_dict()

    for analyzed_run in analyzed_runs:
        for run in analyzed_run["runs"]:
            deal_count[0] += run["analytics"]["deals"]["devil"]
            deal_count[1] += run["analytics"]["deals"]["angel"]
            for bucket in deal_chances:
                deal_chances[bucket]["hit"] += run["analytics"]["dealChance"][bucket][
                    "hit"
                ]
                deal_chances[bucket]["miss"] += run["analytics"]["dealChance"][bucket][
                    "miss"
                ]

    deal_figure = stacked_bar_pie_chart(
        buckets_1=[
            "0% - 25%",
            "26% - 50%",
            "51% - 75%",
            "76% - 100%",
        ],
        xLabel_1="Deal Chance",
        yLabel_1="Count",
        values_bottom_1=[deal_chances[bucket]["hit"] for bucket in deal_chances],
        values_upper_1=[deal_chances[bucket]["miss"] for bucket in deal_chances],
        color_bottom_1="green",
        color_upper_1="red",
        title_1="Hit or Miss of a deal based on chance",
        legend_bottom_1="Hit",
        legend_upper_1="Miss",
        values_2=deal_count,
        labels_2=["Devil", "Angel"],
        colors_2=["dimgray", "gold"],
        title_2="Deal type and count",
        title=f"Devil and Angel Deals ({get_month_header(analyzed_runs[0]["date"])})",
    )

    output_file = join(output_path, "deals.png")
    deal_figure.savefig(output_file, transparent=True)
    logger.info(f"Deal plot saved to {output_file}")



from isaac_analyzer.logging import getLogger
from isaac_analyzer.plotting.single_chart import pie_chart
from os.path import join
from isaac_analyzer.utils import get_month_header

logger = getLogger(__name__)


def get_picture_type(run):
    for floor in run["floors"]:
        if floor["bossRoom"]["boss"] == "Mom":
            for item in floor["bossRoom"]["items"]:
                if item["taken"]:
                    return item["name"]

        if "bossRoomXL" in floor and floor["bossRoomXL"]["boss"] == "Mom":
            for item in floor["bossRoomXL"]["items"]:
                if item["taken"]:
                    return item["name"]


def generate_picture_type_plot(analyzed_runs, output_path):
    picture_type = [0, 0]

    for analyzed_run in analyzed_runs:
        for run in analyzed_run["runs"]:
            if run["analytics"]["picture_type"] == "The Polaroid":
                picture_type[0] += 1
            elif run["analytics"]["picture_type"] == "The Negative":
                picture_type[1] += 1

    picture_figure = pie_chart(
        values=picture_type,
        labels=["The Polaroid", "The Negative"],
        colors=["lightskyblue", "saddlebrown"],
        title=f"Polaroid vs Negative ({get_month_header(analyzed_runs[0]["date"])})",
    )

    output_file = join(output_path, "picture_type.png")
    picture_figure.savefig(output_file, transparent=True)
    logger.info(f"Picture type plot saved to {output_file}")



from re import search
from datetime import datetime
from isaac_analyzer.yaml_loader import load_yaml
from isaac_analyzer.items import get_by_id
from isaac_analyzer.logging import getLogger

logger = getLogger(__name__)


def parse_file_name(file_path):
    logger.debug(f"Parsing file name: {file_path}")
    match = search(r"\d{4}-\d{2}-\d{2}_\d+", file_path)
    if not match:
        logger.error(f"Filename {file_path} does not match the expected format.")
        raise ValueError(f"Filename {file_path} does not match the expected format.")

    date_str, run_number = match.group().split("_")
    date = datetime.strptime(date_str, "%Y-%m-%d").date()
    logger.debug(f"Parsed date: {date}, run_number: {run_number}")
    return date, int(run_number)


def parse_run(raw_run, index):
    logger.debug(f"Parsing run {index}: {raw_run}")
    output = {
        "number": index,
        "type": raw_run["type"],
        "seed": raw_run["seed"],
        "character": raw_run["character"],
        "ending": raw_run["ending"],
        "floors": [parse_floor(floor) for floor in raw_run["floors"]],
    }

    if raw_run["character"] == "Eden" and "startingItems" not in raw_run:
        output["victoryLap"] = True

    if "startingItems" in raw_run:
        output["startingItems"] = parse_items(raw_run["startingItems"])

    logger.debug(f"Parsed run data: {output}")
    return output


def parse_floor(floor):
    logger.debug(f"Parsing floor: {floor}")
    output = {
        "number": floor["number"],
        "type": floor["type"],
        "bossRoom": parse_boss_room(floor["bossRoom"]),
    }

    output["curse"] = bool(floor["curse"])
    if floor["curse"]:
        output["curseType"] = floor["curse"]

    if "itemRoom" in floor:
        output["itemRoom"] = parse_item_room(floor["itemRoom"])

    if "shop" in floor:
        output["shop"] = parse_shop(floor["shop"])

    if "deal" in floor:
        output["deal"] = parse_deal(floor["deal"])

    if "items" in floor:
        output["items"] = parse_items(floor["items"])

    if "bossRoomXL" in floor:
        output["bossRoomXL"] = parse_boss_room(floor["bossRoomXL"])

    if "specialRooms" in floor:
        output["specialRooms"] = parse_special_rooms(floor["specialRooms"])

    logger.debug(f"Parsed floor data: {output}")
    return output


def parse_special_rooms(special_rooms):
    logger.debug(f"Parsing special rooms: {special_rooms}")
    return [parse_special_room(room) for room in special_rooms]


def parse_special_room(special_room):
    logger.debug(f"Parsing special room: {special_room}")
    output = {
        "type": special_room["type"],
        "visited": special_room["visited"],
    }

    if "items" in special_room:
        output["items"] = parse_items(special_room["items"])

    if "number" in special_room:
        output["number"] = special_room["number"]

    logger.debug(f"Parsed special room data: {output}")
    return output


def parse_item_room(item_room):
    logger.debug(f"Parsing item room: {item_room}")
    output = {"visited": item_room["visited"]}

    if item_room["visited"]:
        output.update(
            {
                "rerollMachine": item_room["rerollMachine"],
                "items": parse_items(item_room["items"]),
            }
        )

    logger.debug(f"Parsed item room data: {output}")
    return output


def parse_boss_room(boss_room):
    logger.debug(f"Parsing boss room: {boss_room}")
    output = {"visited": boss_room["visited"]}

    if boss_room["visited"]:
        output.update(
            {
                "boss": boss_room["boss"],
                "champion": boss_room["champion"],
            }
        )
        if "items" in boss_room:
            output["items"] = parse_items(boss_room["items"])

    logger.debug(f"Parsed boss room data: {output}")
    return output


def parse_shop(shop):
    logger.debug(f"Parsing shop: {shop}")
    output = {"visited": shop["visited"]}

    if shop["visited"]:
        if "boss" not in shop:
            output["rerollMachine"] = (shop["rerollMachine"],)
            if "items" in shop:
                output["items"] = parse_items(shop["items"])
        else:
            output["boss"] = shop["boss"]
            if "items" in shop:
                output["items"] = parse_items(shop["items"])

    logger.debug(f"Parsed shop data: {output}")
    return output


def parse_deal(deal):
    logger.debug(f"Parsing deal: {deal}")
    output = {
        "type": deal["type"],
        "chance": deal["chance"],
    }

    if "visited" in deal:
        output["visited"] = deal["visited"]

    if "items" in deal:
        output["items"] = parse_items(deal["items"])

    if "boss" in deal:
        output["boss"] = deal["boss"]

    logger.debug(f"Parsed deal data: {output}")
    return output


def parse_items(items):
    logger.debug(f"Parsing items: {items}")
    return [parse_item(item["id"], item["taken"]) for item in items]


def parse_item(id, taken):
    logger.debug(f"Parsing item with id {id}, taken: {taken}")
    item = get_by_id(id)
    item["taken"] = taken
    logger.debug(f"Parsed item data: {item}")
    return item


def load_run_file(file_path):
    logger.info(f"Loading run file: {file_path}")
    raw_file = load_yaml(file_path)
    date, run_number = parse_file_name(file_path)

    run_data = {
        "date": date,
        "run_number": run_number,
        "runs": [parse_run(run, index) for index, run in enumerate(raw_file["runs"])],
    }

    logger.debug(f"Loaded run file data: {run_data}")
    return run_data



from datetime import datetime


def add_item_analysis_dicts(dict1, dict2):
    result = {"quality": {}, "type": {}}

    # Sum the 'quality' part of the dictionaries
    for key in dict1["quality"]:
        result["quality"][key] = {
            "ignored": dict1["quality"][key]["ignored"]
            + dict2["quality"][key]["ignored"],
            "taken": dict1["quality"][key]["taken"] + dict2["quality"][key]["taken"],
        }

    # Sum the 'type' part of the dictionaries
    for key in dict1["type"]:
        result["type"][key] = {
            "ignored": dict1["type"][key]["ignored"] + dict2["type"][key]["ignored"],
            "taken": dict1["type"][key]["taken"] + dict2["type"][key]["taken"],
        }

    return result


def get_month_header(date: datetime):
    return date.strftime("%B %Y")


def add_dicts(dict1, dict2):
    result_dict = {}
    for key in dict1:
        result_dict[key] = dict1[key] + dict2[key]
    return result_dict


def calculate_percentages(values_bottom, values_upper, buckets):
    totals = [values_bottom[i] + values_upper[i] for i in range(len(buckets))]
    percentage_bottom = [
        values_bottom[i] / totals[i] * 100 if totals[i] != 0 else 0
        for i in range(len(buckets))
    ]
    percentage_upper = [
        values_upper[i] / totals[i] * 100 if totals[i] != 0 else 0
        for i in range(len(buckets))
    ]
    return percentage_bottom, percentage_upper


def sort_keys_per_day_dict(data):
    sorted_keys = sorted(
        data.keys(), key=lambda k: (datetime.strptime(k[:10], "%Y-%m-%d"), k[11:])
    )
    sorted_dict = {k: data[k] for k in sorted_keys}
    return sorted_dict


def get_empty_item_analysis_dict():
    return {
        "quality": {
            "0": {"ignored": 0, "taken": 0},
            "1": {"ignored": 0, "taken": 0},
            "2": {"ignored": 0, "taken": 0},
            "3": {"ignored": 0, "taken": 0},
            "4": {"ignored": 0, "taken": 0},
        },
        "type": {
            "active": {"ignored": 0, "taken": 0},
            "passive": {"ignored": 0, "taken": 0},
        },
    }


def get_empty_curses_dict():
    return {
        "Curse of the Blind": 0,
        "Curse of Darkness": 0,
        "Curse of the Lost": 0,
        "Curse of the Maze": 0,
        "Curse of the Unknown": 0,
        "Curse of the Labyrinth": 0,
        "Curse of the Cursed": 0,
        "Curse of the Giant": 0,
        "No Curse": 0,
        "Total curses": 0,
    }


def get_empty_deal_chance_dict():
    return {
        "<0.25": {"hit": 0, "miss": 0},
        "<0.50": {"hit": 0, "miss": 0},
        "<0.75": {"hit": 0, "miss": 0},
        ">0.75": {"hit": 0, "miss": 0},
    }


def get_empty_shop_details_dict():
    return {"visited_boss": 0, "visited": 0, "skipped": 0}


def get_empty_shop_items_dict():
    return {
        "shop_usage": {"used": 0, "ignored": 0},
        "quality": {"all": 0, "taken": 0, "ignored": 0},
    }


def get_empty_itemRoom_items_dict():
    return {
        "type": {"active": 0, "passive": 0},
        "quality": {"all": 0, "taken": 0, "ignored": 0},
    }


def get_empty_boss_count_dict():
    return {
        "Monstro": 0,
        "Gemini": 0,
        "Steven": 0,
        "Dingle": 0,
        "Gurglings": 0,
        "Larry Jr.": 0,
        "The Duke of Flies": 0,
        "Widow": 0,
        "Blighted Ovum": 0,
        "The Haunt": 0,
        "Pin": 0,
        "Famine": 0,
        "Fistula": 0,
        "Chub": 0,
        "C.H.A.D.": 0,
        "Gurdy": 0,
        "Mega Fatty": 0,
        "Mega Maw": 0,
        "Gurdy Jr.": 0,
        "Peep": 0,
        "The Husk": 0,
        "The Hollow": 0,
        "Carrion Queen": 0,
        "Dark One": 0,
        "Polycephalus": 0,
        "The Wretched": 0,
        "Pestilence": 0,
        "Monstro II": 0,
        "Gish": 0,
        "The Cage": 0,
        "The Gate": 0,
        "Loki": 0,
        "The Adversary": 0,
        "The Bloat": 0,
        "Mask of Infamy": 0,
        "War": 0,
        "Blastocyst": 0,
        "Mama Gurdy": 0,
        "Scolex": 0,
        "Mr. Fred": 0,
        "Lokii": 0,
        "Daddy Long Legs": 0,
        "Triachnid": 0,
        "Teratoma": 0,
        "Death": 0,
        "Conquest": 0,
        "Headless Horseman": 0,
        "The Fallen": 0,
        "Dangle": 0,
        "Turdlings": 0,
        "Little Horn": 0,
        "Rag Man": 0,
        "The Stain": 0,
        "The Forsaken": 0,
        "The Frail": 0,
        "Brownie": 0,
        "Big Horn": 0,
        "Rag Mega": 0,
        "Sisters Vis": 0,
        "The Matriarch": 0,
        "Baby Plum": 0,
        "Bumbino": 0,
        "Reap Creep": 0,
        "The Pile": 0,
        "The Rainmaker": 0,
        "Min-Min": 0,
        "Lil Blub": 0,
        "Wormwood": 0,
        "Clog": 0,
        "Colostomia": 0,
        "Turdlet": 0,
        "Tuff Twins": 0,
        "Hornfel": 0,
        "Great Gideon": 0,
        "Singe": 0,
        "The Shell": 0,
        "Clutch": 0,
        "The Siren": 0,
        "The Heretic": 0,
        "The Visage": 0,
        "The Horny Boys": 0,
        "Chimera": 0,
        "The Scourge": 0,
        "Rotgut": 0,
        "Ultra Famine": 0,
        "Ultra Pestilence": 0,
        "Ultra War": 0,
        "Ultra Death": 0,
        "Mom": 0,
        "Mom's Heart": 0,
        "It Lives": 0,
        "Satan": 0,
        "Isaac": 0,
        "The Lamb": 0,
        "???": 0,
        "Mega Satan": 0,
        "Hush": 0,
        "Delirium": 0,
        "Mother": 0,
        "Dogma": 0,
        "The Beast": 0,
        "Ultra Greed": 0,
        "Ultra Greedier": 0,
    }



from isaac_analyzer.logging import getLogger
from os.path import join
from isaac_analyzer.utils import get_empty_boss_count_dict, add_dicts, get_month_header
from isaac_analyzer.plotting.single_chart import hbars

logger = getLogger(__name__)


def get_boss_type(run):
    bosses = get_empty_boss_count_dict()

    for floor in run["floors"]:
        if (
            "bossRoom" in floor
            and floor["bossRoom"]["visited"]
            and floor["bossRoom"]["boss"] != "None"
        ):
            bosses[floor["bossRoom"]["boss"]] += 1
        if (
            "bossRoomXL" in floor
            and floor["bossRoomXL"]["visited"]
            and floor["bossRoom"]["boss"] != "None"
        ):
            bosses[floor["bossRoomXL"]["boss"]] += 1

    return bosses


def generate_boss_plot(analyzed_runs, output_path):
    bosses = get_empty_boss_count_dict()

    for analyzed_run in analyzed_runs:
        for run in analyzed_run["runs"]:
            bosses = add_dicts(bosses, run["analytics"]["bosses"])

    del bosses["???"]
    del bosses["Isaac"]
    del bosses["It Lives"]
    del bosses["Mom"]
    del bosses["Satan"]
    del bosses["The Lamb"]

    top_bosses = dict(
        sorted(bosses.items(), key=lambda item: item[1], reverse=True)[:12]
    )
    boss_figure = hbars(
        values=[top_bosses[boss] for boss in top_bosses],
        labels=top_bosses.keys(),
        x_label="Count",
        title=f"Top Bosses ({get_month_header(analyzed_runs[0]["date"])})",
    )

    output_file = join(output_path, "boss.png")
    boss_figure.savefig(output_file, transparent=True)
    logger.info(f"Deal plot saved to {output_file}")
